const vector0 = new THREE.Vector(0,0,0)

class MeshGroup {

	constructor(){
		this._meshes = []
		this._geometry = null
		this._instance = null
	}

	addMesh(mesh){
		if(this._meshes.length === 0 ){
			this._geometry = mesh.geometry
		} else if ( mesh.geometry !== this._geometry ) {
			console.warn('wrong geometry')
			return false
		}
		if(this._meshes.indexOf(mesh) === -1) {
			this.meshes.push(mesh)
			return true
		} else {
			return false
		}
	}

	compute(){
		if(this._instance) 
			this._instance.dispose()

		if( !this._geometry ) 
			return 

		this._instance = new THREE.InstancedMesh(
			this._geometry,
			this._meshes[0].material, //not sure what you would want to do here actually
			this._meshes.length,
			true, //always dynamic i guess?
		)
	}

	update(){
		this._meshes.forEach( (mesh,meshIndex) =>{
			this._instance.setQuaternionAt(meshIndex, mesh.quaternion)
			this._instance.setPositionAt(meshIndex, mesh.position)
			this._instance.setScaleAt(meshIndex, mesh.scale)
		})
	}

}

class SceneInstancer {
	constructor(){
		this._instances = {}
		this._instanceScene = new THREE.Scene()
	}

	dispose(){
		Object.keys(this._instances).forEach(instanceKey => {
			this._instanceScene.remove(this._instances[instanceKey])
			this._instances[instanceKey].dispose()
		})
		this._instances = {}	
	}

	setScene( scene ){
		this.dispose()
		this._scene = scene

		scene.traverse( object => {
			if( object instanceof THREE.Mesh ){

				const { geometry } = object
				let instance = this._instances[geometry.uuid]
				
				if(!instance){
					instance = this._instances[geometry.uuid] = new MeshGroup()
				} 
				
				instance.addMesh( object ) 
				object.visible = false

			}
		})

		this._eachInstance(instance=> {
			instance.compute()
			this._instanceScene.add(instance)
		})

	}

	getBatchedScene(){
		return this._instanceScene
	}

	_eachInstance(cb){
		Object.keys(this._instances).forEach(instanceKey=>{
			cb(this._instances[instanceKey])
		})
	}

	update(){
		this._eachInstance(instance=> instance.update())
	}
}

//usage:

function init(){
	//...

	//done setting up your scene graph:

	this.sceneBatcher = new SceneInstancer()

	this.sceneBatcher.setScene( myScene )

}

function myUpdateLoop(){

	myObject.position.copy( myLogic() )

	myCamera.rotation.copy( myController() )

	//etc...

	this.sceneBatcher.update() //copies your stuff into instances

}

function myRenderLoop(){
	myRenderer.render( this.sceneBatcher.getBatchedScene(), myCamera)
}